#include "Ruler.h"
#include <sstream>

Ruler::Ruler(UI *ui, int *lastKey) {
	this->ui = ui;
	this->lastKey = lastKey;
}

void Ruler::update() {
	focalLength = estimateFocalLength(HFOV);
	distance = estimateDistance(rect);
	inputUpdate();
	updateUI();
}

float Ruler::estimateFocalLength(float HFOV) {
	float fl = (ui->getWidth()/2.0) * (cos((HFOV * M_PI)/(180*2.0))/sin(HFOV*M_PI/(180*2.0)));
	return fl;
}

void Ruler::updateUI() {
	if (inputtingVal) {
		cv::putText(*(ui->drawnFrame()), ("Given width: " + std::to_string(givenWidth)), confTextPos, cv::FONT_HERSHEY_PLAIN, 0.7, textFontColor, 2);
	} else {
		cv::putText(*(ui->drawnFrame()), ("Given width: " + std::to_string(givenWidth)), confTextPos, cv::FONT_HERSHEY_PLAIN, 0.7, textFontColor);
	}
	cv::putText(*(ui->drawnFrame()), ("distance: " + std::to_string(distance) + "cm"), distanceTextPos, cv::FONT_HERSHEY_PLAIN, 0.7, textFontColor);

	std::cout << "focal length: " + std::to_string(focalLength) << std::endl;
}

void Ruler::toggleInput() {
	inputtingVal = !inputtingVal;
	if (!inputtingVal) {
		numBuild.clear();
	} else {
		givenWidth = 0;
	}
}

int Ruler::estimateDistance(cv::RotatedRect rect) {
	if (rect.size.area() > 0) {
		return (givenWidth*focalLength) / rect.size.width;
	} else {
		return 0;
	}
}

void Ruler::inputUpdate() {
	if (*lastKey == 13) {
		toggleInput();
	}
	if (inputtingVal) {
		if (*lastKey >= 48 && *lastKey <= 57) {
			char val = '0' + (*lastKey - 48);
			numBuild.push_back(val);
		}
		if (numBuild.size() > 0) {
			std::string numString{numBuild.begin(), numBuild.end()};
			givenWidth = std::stof(numString);
		}
	}
}

void Ruler::updateRectangle(cv::RotatedRect rect) {
	this->rect = rect;
}

void Ruler::componentSetup() {

}

Ruler::~Ruler() {
	// TODO Auto-generated destructor stub
}

