#include <iostream>

#include "Rectangle/ContourAnalyzer.h"
#include "webcam/Webcam.h"
#include "UI.h"
#include "Rectangle/Confidence.h"
#include "Rectangle/BoundTracker.h"
#include "Rectangle/Ruler.h"
enum State {
	detection, track, cross_check
};

State state = detection;
Webcam webcam = 0;
int *lastKey = new int(0);
UI *ui = new UI(lastKey);
ContourAnalyzer ca { ui, lastKey };
Confidence conf { lastKey, ui, 6, 0.78};
BoundTracker tracker { lastKey, ui };
cv::Point statusTextPos {5, 15};
cv::Scalar textFontColor { 20, 20, 20};
cv::Scalar rectangleBoundColor { 255, 0, 0 };
Ruler ruler{ui, lastKey};

cv::RotatedRect subject;
unsigned frameCount = 0;
void run() {
	switch (state) {
	case detection:
		cv::putText(*(ui->drawnFrame()), "Looking for rectangle...",
				statusTextPos, cv::FONT_HERSHEY_PLAIN, 0.8, textFontColor);
		ca.analyze();
		ca.convertContoursToBounds();
		if (conf.check(ca.getBounds())) {
			cv::Rect largest;
			std::vector<cv::Rect> rects = conf.rectangleBounds();
			for (unsigned i = 0; i < rects.size(); i++) {
				if (rects[i].area() > largest.area()) {
					largest = rects[i];
				}
			}

			tracker.init(largest);
			state = track;
		}
		break;

	case track:
	{
		int growBy = 30;
		cv::Rect scaledSearchArea;
		cv::putText(*(ui->drawnFrame()), "Tracking...", statusTextPos, cv::FONT_HERSHEY_PLAIN, 0.8, textFontColor);
		if (!tracker.track()) {
			state = cross_check;
		}
		scaledSearchArea = tracker.getROI();
		scaledSearchArea += cv::Size{growBy*2, growBy*2};
		scaledSearchArea.x -= growBy;
		scaledSearchArea.y -= growBy;

		scaledSearchArea.x = std::max(scaledSearchArea.x, 0);
		scaledSearchArea.y = std::max(scaledSearchArea.y, 0);

		scaledSearchArea.x = std::min(scaledSearchArea.x, ui->getWidth()-scaledSearchArea.width -1);
		scaledSearchArea.y = std::min(scaledSearchArea.y, ui->getHeight()-scaledSearchArea.height -1);

		ca.analyze(scaledSearchArea);

		ca.convertContoursToBounds();
		ca.convertContoursToRotatedBounds();

		if (ca.getBounds().size() > 0) {
			cv::Rect largest;
			unsigned index;
			for (unsigned i = 0; i < ca.getBounds().size(); i++) {
				if (ca.getBounds()[i].area() > largest.area()) {
					largest = ca.getBounds()[i];
					index = i;
				}
			}

			ruler.updateRectangle(ca.getRotatedBounds()[index]);

			largest.x += scaledSearchArea.x;
			largest.y += scaledSearchArea.y;
			tracker.init(largest);
		}
		cv::rectangle(*(ui->drawnFrame()), tracker.getROI(), cv::Scalar {255, 0, 0}, 2);

	}
	break;

	case cross_check:
		cv::putText(*(ui->drawnFrame()), "Cross track...", statusTextPos,
				cv::FONT_HERSHEY_PLAIN, 0.8, textFontColor);
		ca.analyze();
		ca.convertContoursToBounds();
		tracker.track();

		std::vector<cv::Rect> rects = ca.getBounds();
		cv::Rect2d largest;
		if (rects.size() > 0) {
			for (unsigned i = 0; i < rects.size(); i++) {
				if (rects[i].area() > largest.area()) {
					largest = rects[i];
				}
			}

			unsigned largestArea = tracker.getROI().area() > largest.area() ? tracker.getROI().area() : largest.area();

			if ((tracker.getROI() | largest).area() <= largestArea * (1.25)) {
				tracker.init(largest);
				state = track;
				frameCount = 0;
			} else {
				frameCount++;
				if (frameCount > 30) {
					state = detection;
					frameCount = 0;
				}
			}
			cv::rectangle(*(ui->drawnFrame()), largest, cv::Scalar { 255, 0, 0 }, 1);
		} else {
			frameCount++;
			if (frameCount > 30) {
				state = detection;
				frameCount = 0;
			}
		}

		cv::rectangle(*(ui->drawnFrame()), tracker.getROI(), cv::Scalar { 255, 0, 0 }, 2);

		break;
	}

	ruler.update();
	ui->render();
}

int main(int argc, char** argv) {
	while (true) {
		webcam.update();
		cv::Mat frame = webcam.getFrame();
		ui->setOriginalFrame(frame);

		*lastKey = cv::waitKey(33);
		if (*lastKey > 0) {
			std::cout << "key pressed: " << *lastKey << std::endl;
			if (*lastKey == 27) {
				break;
			}
		}
		run();
		*lastKey = -1;
	}

	delete ui;
	delete lastKey;
	cv::destroyAllWindows();
	return 0;
}
